using System;
using Microsoft.Devices.Sensors;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Petzold.Xna3D;

namespace Magnetometer3D
{
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        const string appName = "Accelerometer 3D";

        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;
        BasicEffect basicEffect;

        SpriteFont segoe24Font;
        string text;
        Vector2 textPosition;

        Accelerometer accelerometer;
        Pin pin;

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";

            // Frame rate is 30 fps by default for Windows Phone.
            TargetElapsedTime = TimeSpan.FromTicks(333333);

            // Extend battery life under lock.
            InactiveSleepTime = TimeSpan.FromSeconds(1);

            graphics.IsFullScreen = true;
            graphics.PreferredBackBufferWidth = 480;
            graphics.PreferredBackBufferHeight = 800;

            if (Compass.IsSupported)
            {
                accelerometer = new Accelerometer
                {
                    TimeBetweenUpdates = this.TargetElapsedTime
                };
            }
        }

        protected override void OnActivated(object sender, EventArgs args)
        {
            if (accelerometer != null)
            {
                try { accelerometer.Start(); }
                catch { }
            }
            base.OnActivated(sender, args);
        }

        protected override void OnDeactivated(object sender, EventArgs args)
        {
            if (accelerometer != null)
                accelerometer.Stop();

            base.OnDeactivated(sender, args);
        }

        protected override void Initialize()
        {
            base.Initialize();
        }

        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);
            segoe24Font = this.Content.Load<SpriteFont>("Segoe24Font");

            pin = new Pin(this.GraphicsDevice, 0.05f, 0.75f, 0.25f, 16, Color.Silver, Matrix.Identity);

            basicEffect = new BasicEffect(this.GraphicsDevice);
            basicEffect.VertexColorEnabled = true;
            basicEffect.EnableDefaultLighting();
            basicEffect.PreferPerPixelLighting = true;
            basicEffect.View =
                Matrix.CreateLookAt(new Vector3(0, 0, 4),   // position
                                    new Vector3(0, 0, 0),   // target
                                    new Vector3(0, 1, 0));  // up

            basicEffect.Projection =
                Matrix.CreatePerspectiveFieldOfView(MathHelper.ToRadians(45),
                                                    this.GraphicsDevice.Viewport.AspectRatio,
                                                    0.1f, 10);
        }

        protected override void UnloadContent()
        {
        }

        protected override void Update(GameTime gameTime)
        {
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();

            if (accelerometer != null && accelerometer.IsDataValid)
            {
                Vector3 acceleration = accelerometer.CurrentValue.Acceleration;

                text = String.Format("X = {0:F3}\nY = {1:F3}\nZ = {2:F3}",
                                     acceleration.X,
                                     acceleration.Y,
                                     acceleration.Z);
                textPosition = new Vector2(0, this.GraphicsDevice.Viewport.Height -
                                           segoe24Font.MeasureString(text).Y);
                acceleration.Normalize();
                Vector3 axis = Vector3.Cross(acceleration, Vector3.UnitY);

                // Special case for magnetometer equal to (0, 1, 0) or (0, -1, 0)
                if (axis.LengthSquared() == 0)
                    axis = Vector3.UnitX;
                else
                    axis.Normalize();

                float angle = -(float)Math.Acos(Vector3.Dot(Vector3.UnitY, acceleration));
                basicEffect.World = Matrix.CreateFromAxisAngle(axis, angle);
            }
            else
            {
                basicEffect.World = Matrix.Identity;
                text = "";
            }
            base.Update(gameTime);
        }

        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.Navy);

            // 2D text
            spriteBatch.Begin();
            spriteBatch.DrawString(segoe24Font, appName, Vector2.Zero, Color.White);
            spriteBatch.DrawString(segoe24Font, text, textPosition, Color.White);
            spriteBatch.End();

            GraphicsDevice.DepthStencilState = DepthStencilState.Default;
            GraphicsDevice.BlendState = BlendState.Opaque;

            // 3D pin
            foreach (EffectPass effectPass in basicEffect.CurrentTechnique.Passes)
            {
                effectPass.Apply();
                pin.Draw();
            }

            base.Draw(gameTime);
        }
    }
}
